@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at gmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2009-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @copyright 2009-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_proplists).
@@ -45,7 +45,8 @@
 %%% External interface functions
 %%%------------------------------------------------------------------------
 
--type tuplelist() :: list({atom(), any()}).
+-type property() :: {atom(), any()}.
+-export_type([property/0]).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -54,13 +55,14 @@
 %%-------------------------------------------------------------------------
 
 -spec delete_all(Keys :: list(atom()),
-                 List :: tuplelist()) -> tuplelist().
+                 List :: list(property())) ->
+    list(property()).
 
 delete_all([], List) ->
     List;
 
 delete_all([Key | Keys], List)
-    when is_atom(Key), is_list(List) ->
+    when is_atom(Key) ->
     case lists:keytake(Key, 1, List) of
         {value, _, NewList} ->
             delete_all(Keys, NewList);
@@ -75,13 +77,14 @@ delete_all([Key | Keys], List)
 %%-------------------------------------------------------------------------
 
 -spec find_any(Keys :: list(atom()),
-               List :: tuplelist()) -> boolean().
+               List :: list(property())) ->
+    boolean().
 
 find_any([], _) ->
     false;
 
 find_any([Key | Keys], List)
-    when is_atom(Key), is_list(List) ->
+    when is_atom(Key) ->
     case lists:keyfind(Key, 1, List) of
         {Key, _} ->
             true;
@@ -96,10 +99,11 @@ find_any([Key | Keys], List)
 %%-------------------------------------------------------------------------
 
 -spec partition(Key :: atom(),
-                List :: tuplelist()) -> {tuplelist(), tuplelist()}.
+                List :: list(property())) ->
+    {list(property()), list(property())}.
 
 partition(Key, List)
-    when is_atom(Key), is_list(List) ->
+    when is_atom(Key) ->
     lists:partition(fun({K, _}) -> K == Key end, List).
 
 %%-------------------------------------------------------------------------
@@ -109,15 +113,15 @@ partition(Key, List)
 %% @end
 %%-------------------------------------------------------------------------
 
--spec take_values(DefaultList :: tuplelist(),
-                  List :: tuplelist()) -> list().
+-spec take_values(DefaultList :: list(property()),
+                  List :: list(property())) ->
+    list().
 
-take_values(DefaultList, List)
-    when is_list(DefaultList), is_list(List) ->
+take_values(DefaultList, List) ->
     take_values([], DefaultList, List).
 
 take_values(Result, [], List) ->
-    lists:reverse(Result) ++ List;
+    lists:reverse(Result, List);
 
 take_values(Result, [{Key, Default} | DefaultList], List)
     when is_atom(Key) ->
