@@ -25,14 +25,6 @@
 # DEALINGS IN THE SOFTWARE.
 #
 
-import sys, os
-sys.path.append(
-    os.path.sep.join(
-        os.path.dirname(os.path.abspath(__file__))
-               .split(os.path.sep)[:-2] + ['api', 'python']
-    )
-)
-
 from cloudi_c import API, terminate_exception
 from http_req import Task
 
